#include <stdio.h>

#define gint int
#define gchar char
#define BRK_CHARS "() \t\r\n"

/* ref_expander()
   
   Recognizes strings in src of the form "foo:bar:r1c2" and replaces them 
   with a filled in template, such as "(get-cell %ld %ld \"%s\" \"%s\")"
   It also recognizes "foo:r1c1" and "r1c1", passing an empty string
   for the second & third arguments to the template, as appropriate */

typedef enum {
  ASSUME_RnCn, ASSUME_NAME, KNOWN_STRING
} ref_state;

/* works like a FSM: always assume that the upcoming input is a
   "r1c1"-ref unless we can prove otherwise.  If so, assume it's a
   page name, unless we can prove otherwise.  If we've already got a
   page name, assume the first was really a buffer name.  If it ain't
   none of those things, pass it to the output unchanged.  

   start_assumption is the start of the text we're still looking at

   interim_assumption is the start of the text we're looking at but
   haven't categorized yet

   end_assumption is (just past) the end of the text we've looked at
*/

void ref_expander(gchar *src, gchar *dest, gchar *brk, gchar *tmpl) 
{
  ref_state state = ASSUME_RnCn;
  gchar buffer_name[50];  /* FIXME */
  gchar bpage_name[50];   /* FIXME */
  gchar *start_assumption, *interim_assumption, *end_assumption;
  gint c;
  gint row, col;

  strcpy(dest,"never written into\n");

  buffer_name[0] = bpage_name[0] = 0;
  start_assumption = interim_assumption = end_assumption = src;
  while (c = *end_assumption++) {
    switch(state) {
    case KNOWN_STRING:
      if (c == '"')  { 	
	while (start_assumption < end_assumption) 
	  *dest++ = *start_assumption++;
	start_assumption = interim_assumption = end_assumption;
	state = ASSUME_RnCn; 
      } 
      if (c == '\\') { 
	/* consume next character also */
	c = *end_assumption++; 
      }
      break; 

    case ASSUME_RnCn:
      if (c == '"') { state = KNOWN_STRING; break; }
      if ((c == 'r' || c == 'R') && isdigit(*end_assumption)) {
	/* found component 1&2: "Rn" */
	row = strtol(end_assumption, &end_assumption, 10);
	c = *end_assumption++;  /* FIXME: end of string? */
	if ((c == 'c' || c == 'C') && isdigit(*end_assumption)) {
	  /* found component 3&4: "Cn" */
	  col = strtol(end_assumption, &end_assumption, 10);
	  if (*end_assumption && !strchr(brk, *end_assumption)) {
	    /* didn't end with a break character: "RnCnSOMETHING" */
	    state = ASSUME_NAME;
	  } else {
	    /* found all 5 conditions: "RnCn " ! */
	    start_assumption = interim_assumption = end_assumption;
	    dest += sprintf(dest, tmpl, row, col, bpage_name, buffer_name);
	  }
	}
      } else { 	/* didn't find "R" */
	state = ASSUME_NAME;
      }
    case ASSUME_NAME:
      if (c == ':') {  /* end of a buffer or page name */
	int i;
	if (bpage_name[0] == 0) {   /* page name if we don't have one */
	  for (i = 0; interim_assumption < end_assumption; i++) {
	    bpage_name[i] = *interim_assumption++;
	  }
	  bpage_name[i-1] = 0;      /* replace ':' with 0L */
	  state = ASSUME_RnCn;      
	} else {
	  strcpy(buffer_name, bpage_name);
	  for (i = 0; interim_assumption < end_assumption; i++) {
	    bpage_name[i] = *interim_assumption++;
	  }
	  bpage_name[i-1] = 0;
	  state = ASSUME_RnCn;
	}
      } else if (strchr(brk, c)) {
	/* break char => wasn't a name, so punt */
	while (start_assumption < end_assumption) 
	  *dest++ = *start_assumption++;
	start_assumption = interim_assumption = end_assumption;
	state = ASSUME_RnCn;
      } else {
	/* wasn't a ':' or a brk, so keep going */
	break;
      }
    }
  }
  while (start_assumption < end_assumption) 
    *dest++ = *start_assumption++;
}


void simplified_ref_expander(gchar *src, gchar *dest, gchar *brk, gchar *tmpl) 
{
  gchar *bname, *bpname, *rc0, *rc1;
  gchar *first;
  gint row, col;
  gint found;

  while (found = ref_finder(src, brk,
			    &bname, &bpname, &rc0, &rc1,
			    &row, &col)) {

    fprintf(stderr, "bname: %s\n", bname);
    fprintf(stderr, "bpame: %s\n", bpname);
    fprintf(stderr, "rc_0 : %s\n", rc0);
    fprintf(stderr, "rc_1 : %s\n", rc1);

    first = bname?bname:(bpname?bpname:rc0);
    while (src < first) 
      *dest++ = *src++;
    src = rc1;
    if (bpname) rc0[-1] = 0;
    if (bname)  bpname[-1] = 0;
    bname  =  bname?bname:""  ;
    bpname = bpname?bpname:"" ;
    dest += sprintf(dest, tmpl, row, col, bpname, bname);
  }

  while (*dest++ = *src++);
}


int ref_finder(gchar *src, gchar *brk, 
	       gchar **bname, gchar **bpname, gchar **rc_begin, gchar **rc_end,
	       gint *row, gint *col) 
{
  ref_state state = ASSUME_RnCn;
  gchar *start_assumption, *interim_assumption, *end_assumption;
  gint c;

  *bname = *bpname = NULL;

  start_assumption = interim_assumption = end_assumption = src;
  while (c = *end_assumption++) {
    switch(state) {
    case KNOWN_STRING:
      if (c == '"')  { 	
	start_assumption = interim_assumption = end_assumption;
	state = ASSUME_RnCn; 
      } 
      if (c == '\\') {             /* consume next character also */
	end_assumption++; 
      }
      break; 

    case ASSUME_RnCn:
      if (c == '"') { state = KNOWN_STRING;  break; }

      if ((c == 'r' || c == 'R') && isdigit(*end_assumption)) {
	/* found component 1&2: "Rn" */
/* 	if (*bpname == NULL) *bpname = interim_assumption; */
/* 	if (*bname == NULL)  *bname = *bpname;  */
	*rc_begin = interim_assumption;
	*row = strtol(end_assumption, &end_assumption, 10);
	c = *end_assumption++;  /* FIXME: end of string? */
	if ((c == 'c' || c == 'C') && isdigit(*end_assumption)) {
	  /* found component 3&4: "Cn" */
	  *col = strtol(end_assumption, &end_assumption, 10);
	  if (*end_assumption && !strchr(brk, *end_assumption)) {
	    /* didn't end with a break character: "RnCnSOMETHING" */
	    state = ASSUME_NAME;
	  } else {
	    /* found all 5 conditions: "RnCn " ! */
	    *rc_end = end_assumption;
	    return 1;
	  }
	}
      } else { 	/* didn't find "Rn" */
	state = ASSUME_NAME;
      }
    case ASSUME_NAME:
      if (c == ':') {                    /* end of a bname or bpname */
	int i;
	if (*bpname == NULL) {  /* page name if we don't have one yet */
	  *bpname = interim_assumption;
	  interim_assumption = end_assumption;
      } else {                        /* page name if we do have one */
	  *bname = *bpname; 
	  *bpname = interim_assumption;
	  interim_assumption = end_assumption;
      }
	state = ASSUME_RnCn;         
      } else if (strchr(brk, c)) {
	/* break char --> wasn't a name, so punt */
	start_assumption = interim_assumption = end_assumption;
	state = ASSUME_RnCn;
      } else {
	/* wasn't a ':' or a brk, so keep going */
	break;
      }
    }
  }
  /* must not have found any reference */
  bname = bpname = rc_end = rc_begin = NULL;
  return 0;
}



main ()
{
  gchar in[1000];
  gchar out[1000];
  gchar *bname, *bpname, *rc1, *rc2; 
  gint row, col;
  gint found;

  while(!feof(stdin)) {
    fscanf(stdin, "%[^\n]", in);
    fgetc(stdin);
    simplified_ref_expander(in, out, BRK_CHARS, "(get-cell %d %d \"%s\" \"%s\")");
    fprintf(stderr,"res: %s\n", out);

    found = ref_finder(in, BRK_CHARS, 
		       &bname, &bpname, 
		       &rc1, &rc2,
		       &row, &col);

    fflush(stdout);
  }
}
