/* $XConsortium: TableP.h,v 1.7 94/04/17 20:13:00 rws Exp $ */

/*

Copyright (c) 1987  X Consortium

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
X CONSORTIUM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

Except as contained in this notice, the name of the X Consortium shall not be
used in advertising or otherwise to promote the sale, use or other dealings
in this Software without prior written authorization from the X Consortium.

*/


#ifndef __GTK_NGRID_H__
#define __GTK_NGRID_H__

/* make these into enums? */
#define NGRID_TEXT   1
#define NGRID_PIXMAP 2
#define NGRID_EMBED  3

#include <gdk/gdk.h>
#include <gtk/gtk.h>
#include <gtk/gtkwidget.h>

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#define GTK_NGRID(obj)          GTK_CHECK_CAST (obj, gtk_ngrid_get_type (), GtkNGrid)
#define GTK_NGRID_CLASS(klass)  GTK_CHECK_CLASS_CAST (klass, gtk_ngrid_get_type (), GtkGridClass)
#define GTK_IS_NGRID(obj)       GTK_CHECK_TYPE (obj, gtk_ngrid_get_type ())

typedef struct _GtkNGrid                GtkNGrid;
typedef struct _GtkNGridClass           GtkNGridClass;
typedef struct _GtkNGridEntryFeatures   GtkNGridEntryFeatures;

struct _GtkNGridEntryFeatures
{
  gulong format;      /* default to 0 */
  gint fontsize;      /* ignored if -1   */
  GdkGC *gc;          /* ignored if null */
  GdkFont *font;      /* ignored if null */
  GtkWidget *widget;  /* ignored if null */
  guint highlight :1;
  guint free_me : 1;
};

struct _GtkNGridClass
{
  GtkWidgetClass parent_class;
  void (* changed)  (GtkNGrid *ngrid);
  void (* clicked)  (GtkNGrid *ngrid);
};

struct _GtkNGrid {
  GtkWidget widget;                                       /* parent class */
  gint max_row, max_col;                /* last row/col that can be shown */
  gint prot_row, prot_col;               /* num of rows/cols always shown */
  GtkAdjustment *hadjustment;                   /* adjustment for top_col */
  GtkAdjustment *vadjustment;                   /* adjustment for top_row */
  gint top_row, top_col;               /* first unprotected row/col drawn */
  gint sel_top_row, sel_bottom_row;     /* selection */      /* not used yet in ml */
  gint sel_left_col, sel_right_col;     /* selection */      /* not used yet in ml */
  gint point_row, point_col;                        /* location of cursor */
  gint default_width, default_height;      /* default cell size in pixels */
  
  /* query functions that describe cells */
  gint (*col_width)(GtkNGrid *ng, gpointer data, gint col);     
  gint (*row_height)(GtkNGrid *ng, gpointer data, gint row);
  gint (*type)(GtkNGrid *ng, gpointer data, gint row, gint col);
  char *(*text)(GtkNGrid *ng, gpointer data, gint row, gint col);
  GtkNGridEntryFeatures* (*features)(GtkNGrid *ng, gpointer data, gint row, gint col);

  gpointer data;             /* arbitrary user data passed to query funcs */
  guint grid_lines : 1;                               /* draw grid lines? */
  guint visible_cursor : 1;                           /* draw cursor?     */

  /* private state */
  GdkGC *non_gr_exp_gc;                               /* gc for ? */
  GdkGC *xor_gc;                                      /* gc for cursor */
};

guint      gtk_ngrid_get_type   (void);
GtkWidget* gtk_ngrid_new        (gint rows, gint cols) ;
void       gtk_ngrid_size       (GtkNGrid      *grid,
				 gint          width,
				 gint          height) ;

void gtk_ngrid_rowcol_to_coord(GtkNGrid *grid,
			       gint cell_row, gint cell_col,
			       gint *cell_x,  gint *cell_y) ;
void gtk_ngrid_coord_to_rowcol(GtkNGrid *tw,
			       int cur_x, int cur_y,
			       int *cur_row, int *cur_col) ;

GtkAdjustment* gtk_ngrid_get_hadjustment (GtkNGrid   *ngrid) ;
GtkAdjustment* gtk_ngrid_get_vadjustment (GtkNGrid   *ngrid) ;
void           gtk_ngrid_set_hadjustment (GtkNGrid   *ngrid,
					  GtkAdjustment *adjustment) ;
void           gtk_ngrid_set_vadjustment (GtkNGrid   *ngrid,
					  GtkAdjustment *adjustment) ;

void gtk_ngrid_set_point (GtkNGrid *ngrid,
			  gint row, gint col) ;
void gtk_ngrid_get_point (GtkNGrid *ngrid,
			  gint *row, gint *col) ;
gint
gtk_ngrid_expose (GtkWidget      *widget,
		  GdkEventExpose *event) ;


#ifdef __cplusplus
}
#endif /* __cplusplus */


#endif /* __GTH_NGRID_H__ */



